package org.ow2.util.pool.impl.enhanced;

import java.util.concurrent.Executor;

import javax.resource.spi.work.WorkManager;

import org.ow2.util.pool.impl.enhanced.api.IPool;
import org.ow2.util.pool.impl.enhanced.api.thread.IReusableThread;
import org.ow2.util.pool.impl.enhanced.impl.basic.BasicPool;
import org.ow2.util.pool.impl.enhanced.impl.basic.accessmanager.LastUsedAccessManager;
import org.ow2.util.pool.impl.enhanced.impl.thread.managementthread.ManagementThreadReusableThreadFactory;
import org.ow2.util.pool.impl.enhanced.impl.thread.threadpoolexecutor.ExecutorReusableThreadFactory;
import org.ow2.util.pool.impl.enhanced.impl.thread.workmanager.WorkManagerReusableThreadFactory;
import org.ow2.util.pool.impl.enhanced.impl.util.ExecutorProvider;

/**
 * @author Gael Lalire
 */
public final class ReusableThreadPoolFactory {

    private ReusableThreadPoolFactory() {
    }

    /**
     * Use a management thread to create new pool item (Executor). Use a
     * management thread also to call setExpectedSize (ISharedManager).
     * @param poolSize the number of management thread
     */
    public static IPool<IReusableThread> createManagementThreadPool(final int poolSize) {
        return new BasicPool<IReusableThread>(new ManagementThreadReusableThreadFactory(true), poolSize,
                new LastUsedAccessManager<IReusableThread>(), ExecutorProvider.SELF_THREAD_EXECUTOR, null, null);
    }

    /**
     * Use the work manager to create, fetch and record items of pool
     * (Executor). Use the work manager also to call setExpectedSize
     * (ISharedManager).
     * @param workManager the workManager
     * @param limit max call to doSchedule of pool management
     */
    public static IPool<IReusableThread> createWorkManagerThreadPool(final WorkManager workManager, final int limit) {
        return new BasicPool<IReusableThread>(new WorkManagerReusableThreadFactory(workManager), limit,
                new LastUsedAccessManager<IReusableThread>(), ExecutorProvider.SELF_THREAD_EXECUTOR, null, null);
    }

    /**
     * Use the executor to create, fetch and record items of pool (Executor).
     * Use the executor also to call setExpectedSize (ISharedManager).
     * @param executor the executor
     * @param limit max call to execute of pool management
     */
    public static IPool<IReusableThread> createExecutorThreadPool(final Executor executor, final int limit) {
        return new BasicPool<IReusableThread>(new ExecutorReusableThreadFactory(executor), limit,
                new LastUsedAccessManager<IReusableThread>(), ExecutorProvider.SELF_THREAD_EXECUTOR, null, null);
    }

}
