package org.ow2.util.pool.impl.enhanced.callback;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.pool.impl.enhanced.api.basic.IPoolItemRemoveListener;
import org.ow2.util.pool.impl.enhanced.impl.PoolError;
import org.ow2.util.pool.impl.enhanced.internal.stoppableexecution.IArgumentsCallable;
import org.ow2.util.pool.impl.enhanced.internal.stoppableexecution.StoppableExecution;
import org.ow2.util.pool.impl.enhanced.internal.stoppableexecution.StoppedExecutionException;

/**
 * @author Gael Lalire
 * @param <E>
 */
public class RemoveCB<E> extends StoppableExecution<Void> implements IPoolItemRemoveListener<E> {

    private static final Log LOG = LogFactory.getLog(RemoveCB.class);

    public RemoveCB(final IPoolItemRemoveListener<E> removeListener) {
        super(new IArgumentsCallable<Void>() {

            @SuppressWarnings("unchecked")
            public Void call(final Object... args) throws Exception {
                removeListener.poolItemRemoved((E) args[0]);
                return null;
            }

        });
    }

    public void poolItemRemoved(final E poolItem) {
        try {
            call(poolItem);
        } catch (StoppedExecutionException e) {
            LOG.warn("poolItemRemoved failed", e);
        } catch (RuntimeException e) {
            LOG.warn("poolItemRemoved failed", e);
        } catch (Exception e) {
            throw new PoolError("poolItemRemoved could not throw checked exception", e);
        }
    }

}
